/*
 * test_rw_echo.c
 * Test blocking I/O model which is built on top of PortAudio.
 * Echo delayed input to output.
 *
 * Author: Phil Burk, http://www.softsynth.com
 *
 * This program uses the PortAudio Portable Audio Library.
 * For more information see: http://www.audiomulch.com/portaudio/
 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "portaudio.h"
#include "rwblock.h"
#include <string.h>

/*
** Note that many of the older ISA sound cards on PCs do NOT support
** full duplex audio (simultaneous record and playback).
** And some only support full duplex at lower sample rates.
*/
#define SAMPLE_RATE         (22050)
#define NUM_SECONDS            (20)
#define SAMPLES_PER_FRAME       (2)

/* Select whether we will use floats or shorts. */
#if 1
#define SAMPLE_TYPE  paFloat32
typedef float SAMPLE;
#else
#define SAMPLE_TYPE  paInt16
typedef short SAMPLE;
#endif

#define NUM_ECHO_FRAMES   (2*SAMPLE_RATE)
SAMPLE   samples[NUM_ECHO_FRAMES][SAMPLES_PER_FRAME] = {0.0};

/*******************************************************************/
int main(void);
int main(void)
{
	int      i;
	PaError  err;
	RWBL     *rwbl_in;
	RWBL     *rwbl_out;
	int      index;

	printf("Full duplex sound test using PortAudio and RingBuffers\n");
	fflush(stdout);

/* Open simplified blocking I/O layer on top of PortAudio. */
/* Open input first so it can start to fill buffers. */
	err = OpenAudioStream( &rwbl_in, SAMPLE_RATE, SAMPLE_TYPE,
				     (RWBL_READ | RWBL_STEREO) );
	if( err != paNoError ) goto error;
	/* printf("opened input\n");  fflush(stdout); /**/

	err = OpenAudioStream( &rwbl_out, SAMPLE_RATE, SAMPLE_TYPE,
				     (RWBL_WRITE | RWBL_STEREO) );
	if( err != paNoError ) goto error;
	/* printf("opened output\n");  fflush(stdout); /**/

/* Process samples in the foreground. */
	index = 0;
	for( i=0; i<(NUM_SECONDS * SAMPLE_RATE); i++ )
	{
	/* Write old frame of data to output. */
		/* samples[index][1] = (i&256) * (1.0f/256.0f); /* sawtooth */
		WriteAudioStream( rwbl_out, &samples[index][0], 1 );
		
	/* Read one frame of data into sample array for later output. */
		ReadAudioStream( rwbl_in, &samples[index][0], 1 );
		index += 1;
		if( index >= NUM_ECHO_FRAMES ) index = 0;
		
		if( (i & 0xFFFF) == 0 ) printf("i = %d\n", i ); fflush(stdout); /**/
	}

	CloseAudioStream( rwbl_out );
	CloseAudioStream( rwbl_in );

	printf("R/W echo sound test complete.\n" );
	fflush(stdout);
	return 0;

error:
	fprintf( stderr, "An error occured while using PortAudio\n" ); 
	fprintf( stderr, "Error number: %d\n", err );
	fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
	return -1;
}
