/*
 * patest_wire.c
 * Pass input directly to output.
 *
 * Author: Phil Burk, http://www.softsynth.com
 *
 * This program uses the PortAudio Portable Audio Library.
 * For more information see: http://www.audiomulch.com/portaudio/
 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <math.h>
#include "portaudio.h"

/*
** Note that many of the older ISA sound cards on PCs do NOT support
** full duplex audio (simultaneous record and playback).
** And some only support full duplex at lower sample rates.
*/
#define SAMPLE_RATE  (22050)
#define NUM_SECONDS  (2)

#if 1
#define PA_SAMPLE_TYPE  paFloat32
typedef float SAMPLE;
#else
#define PA_SAMPLE_TYPE  paInt16
typedef short SAMPLE;
#endif

typedef struct {
	unsigned int sampsToGo;
	int          debug;
}paTestData;

static int wireCallback(	void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
				        PaTimestamp outTime, void *userData );
				        
/* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*/
static int wireCallback(	void *inputBuffer, void *outputBuffer,
						unsigned long framesPerBuffer,
				        PaTimestamp outTime, void *userData )
{
	paTestData *data = (paTestData*)userData;
	SAMPLE *out = (SAMPLE*)outputBuffer;
	SAMPLE *in = (SAMPLE*)inputBuffer;

	unsigned int i;
	int finished = 0;

	(void) outTime;
	
/* This may get called with NULL inputBuffer during initial setup. */
	if( inputBuffer == NULL )
	{
		data->debug += 1;
		for( i=0; i<framesPerBuffer; i++ )
		{
			*out++ = 0;		/* left */
			*out++ = 0;		/* right */
		}
	}
	else
	{
		for( i=0; i<framesPerBuffer; i++ )
		{
			*out++ = *in++;		/* left */
			*out++ = *in++;		/* right */
		}
	}
		
	if( data->sampsToGo < framesPerBuffer )
	{
		data->sampsToGo = 0;
		finished = 1;
	}
	else
	{
		data->sampsToGo -= framesPerBuffer;
	}
	
	return finished;
}

/*******************************************************************/
int main(void);
int main(void)
{
	PortAudioStream *stream;
	PaError err;
	paTestData data = { 0 };
	int totalSamps;
	int numDevices = Pa_CountDevices();

	data.sampsToGo = totalSamps = NUM_SECONDS * SAMPLE_RATE; /* Play for a few seconds. */

	err = Pa_Initialize();
	if( err != paNoError ) goto error;

	err = Pa_OpenStream(
				&stream,
				Pa_GetDefaultInputDeviceID(), /* default output device */
				2,               /* stereo input */
				PA_SAMPLE_TYPE,	
				NULL,
				Pa_GetDefaultOutputDeviceID(), /* default output device */
				2,               /* stereo output */
				PA_SAMPLE_TYPE, 
				NULL,
				SAMPLE_RATE,
				256,            /* frames per buffer */
				0,               /* number of buffers, if zero then use default minimum */
				paClipOff,       /* we won't output out of range samples so don't bother clipping them */
				wireCallback,
				&data );
	if( err != paNoError ) goto error;

	err = Pa_StartStream( stream );
	if( err != paNoError ) goto error;

	printf("Full duplex sound test in progress.\n"); fflush(stdout);
	while( Pa_StreamActive( stream ) ) Pa_Sleep(100); /* SPIN! */

	err = Pa_CloseStream( stream );
	if( err != paNoError ) goto error;

	Pa_Terminate();
	printf("Full duplex sound test complete. debug = %d\n", data.debug ); fflush(stdout);
	return 0;

error:
	Pa_Terminate();
	fprintf( stderr, "An error occured while using the portaudio stream\n" ); 
	fprintf( stderr, "Error number: %d\n", err );
	fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
	return -1;
}
